@@ -22,7 +22,6 @@ void CommandData::Init()
   *ArcNameW=0;
   FileLists=false;
   NoMoreSwitches=false;
-  TimeConverted=false;
 
   FileArgs=new StringList;
   ExclArgs=new StringList;
@@ -51,7 +50,7 @@ void CommandData::ParseArg(char *Arg,wchar *ArgW)
     if (Arg[1]=='-')
       NoMoreSwitches=true;
     else
-      ProcessSwitch(&Arg[1]);
+      ProcessSwitch(&Arg[1],(ArgW!=NULL && *ArgW!=0 ? &ArgW[1]:NULL));
   else
     if (*Command==0)
     {
@@ -81,8 +80,8 @@ void CommandData::ParseArg(char *Arg,wchar *ArgW)
       }
       else
       {
-        int Length=strlen(Arg);
-        char EndChar=Arg[Length-1];
+        size_t Length=strlen(Arg);
+        char EndChar=Length==0 ? 0:Arg[Length-1];
         char CmdChar=etoupper(*Command);
         bool Add=strchr("AFUM",CmdChar)!=NULL;
         bool Extract=CmdChar=='X' || CmdChar=='E';
@@ -113,7 +112,7 @@ void CommandData::ParseArg(char *Arg,wchar *ArgW)
                 Charset=RCH_OEM;
 #endif
 
-              ReadTextFile(Arg+1,FileArgs,false,true,Charset,true,true);
+              ReadTextFile(Arg+1,FileArgs,false,true,Charset,true,true,true);
             }
             else
               if (Found && FileData.IsDir && Extract && *ExtrPath==0)
@@ -162,6 +161,8 @@ bool CommandData::IsConfigEnabled(int argc,char *argv[])
   for (int I=1;I<argc;I++)
     if (IsSwitch(*argv[I]))
     {
+      if (stricomp(&argv[I][1],"-")==0)
+        break;
       if (stricomp(&argv[I][1],"cfg-")==0)
         ConfigEnabled=false;
 #ifndef GUI
@@ -175,7 +176,7 @@ bool CommandData::IsConfigEnabled(int argc,char *argv[])
 #endif
       if (strnicomp(&argv[I][1],"sc",2)==0)
       {
-        // process -cs before reading any file lists
+        // Process -sc before reading any file lists.
         ProcessSwitch(&argv[I][1]);
       }
     }
@@ -226,7 +227,7 @@ void CommandData::ProcessSwitchesString(char *Str)
 
 
 #if !defined(SFX_MODULE)
-void CommandData::ProcessSwitch(char *Switch)
+void CommandData::ProcessSwitch(char *Switch,wchar *SwitchW)
 {
 
   switch(etoupper(Switch[0]))
@@ -368,6 +369,9 @@ void CommandData::ProcessSwitch(char *Switch)
             strncpyz(GenerateMask,Switch+2,ASIZE(GenerateMask));
           }
           break;
+        case 'I':
+          IgnoreGeneralAttr=true;
+          break;
         case 'N': //reserved for archive name
           break;
         case 'O':
@@ -375,6 +379,8 @@ void CommandData::ProcessSwitch(char *Switch)
           break;
         case 'P':
           strcpy(ArcPath,Switch+2);
+          if (SwitchW!=NULL && *SwitchW!=0)
+            strcpyw(ArcPathW,SwitchW+2);
           break;
         case 'S':
           SyncFiles=true;
@@ -408,6 +414,9 @@ void CommandData::ProcessSwitch(char *Switch)
         case '-':
           Overwrite=OVERWRITE_NONE;
           break;
+        case 0:
+          Overwrite=OVERWRITE_FORCE_ASK;
+          break;
         case 'R':
           Overwrite=OVERWRITE_AUTORENAME;
           break;
@@ -439,7 +448,7 @@ void CommandData::ProcessSwitch(char *Switch)
           Recurse=RECURSE_ALWAYS;
           break;
         case '-':
-          Recurse=0;
+          Recurse=RECURSE_DISABLE;
           break;
         case '0':
           Recurse=RECURSE_WILDCARDS;
@@ -480,7 +489,7 @@ void CommandData::ProcessSwitch(char *Switch)
             Charset=RCH_OEM;
 #endif
 
-          ReadTextFile(Switch+2,Args,false,true,Charset,true,true);
+          ReadTextFile(Switch+2,Args,false,true,Charset,true,true,true);
         }
         else
           Args->AddString(Switch+1);
@@ -639,7 +648,7 @@ void CommandData::ProcessSwitch(char *Switch)
 #ifdef PACK_SMP
         case 'T':
           Threads=atoi(Switch+2);
-          if (Threads>MaxSearchThreads)
+          if (Threads>16)
             BadSwitch(Switch);
           else
           {
@@ -676,16 +685,16 @@ void CommandData::ProcessSwitch(char *Switch)
           break;
         default:
           {
-            Int64 NewVolSize=atoil(&Switch[1]);
+            int64 NewVolSize=atoil(&Switch[1]);
 
             if (NewVolSize==0)
-              NewVolSize=INT64ERR;
+              NewVolSize=INT64NDF; // Autodetecting volume size.
             else
               switch (Switch[strlen(Switch)-1])
               {
                 case 'f':
                 case 'F':
-                  switch(int64to32(NewVolSize))
+                  switch(NewVolSize)
                   {
                     case 360:
                       NewVolSize=362496;
@@ -794,7 +803,10 @@ void CommandData::ProcessSwitch(char *Switch)
             break;
           case 'C':
             {
-              RAR_CHARSET rch;
+              // Switch is already found bad, avoid reporting it several times.
+              bool AlreadyBad=false;
+
+              RAR_CHARSET rch=RCH_DEFAULT;
               switch(etoupper(Switch[2]))
               {
                 case 'A':
@@ -808,24 +820,27 @@ void CommandData::ProcessSwitch(char *Switch)
                   break;
                 default :
                   BadSwitch(Switch);
+                  AlreadyBad=true;
                   break;
               };
-              if (Switch[3]==0)
-                CommentCharset=FilelistCharset=rch;
-              else
-                for (int I=3;Switch[I]!=0;I++)
-                  switch(etoupper(Switch[I]))
-                  {
-                    case 'C':
-                      CommentCharset=rch;
-                      break;
-                    case 'L':
-                      FilelistCharset=rch;
-                      break;
-                    default:
-                      BadSwitch(Switch);
-                      break;
-                  }
+              if (!AlreadyBad)
+                if (Switch[3]==0)
+                  CommentCharset=FilelistCharset=rch;
+                else
+                  for (int I=3;Switch[I]!=0 && !AlreadyBad;I++)
+                    switch(etoupper(Switch[I]))
+                    {
+                      case 'C':
+                        CommentCharset=rch;
+                        break;
+                      case 'L':
+                        FilelistCharset=rch;
+                        break;
+                      default:
+                        BadSwitch(Switch);
+                        AlreadyBad=true;
+                        break;
+                    }
             }
             break;
 
@@ -908,43 +923,59 @@ void CommandData::OutTitle()
 #endif
 
 
+inline bool CmpMSGID(MSGID i1,MSGID i2)
+{
+#ifdef MSGID_INT
+  return(i1==i2);
+#else
+  // If MSGID is const char*, we cannot compare pointers only.
+  // Pointers to different instances of same strings can differ,
+  // so we need to compare complete strings.
+  return(strcmp(i1,i2)==0);
+#endif
+}
+
 void CommandData::OutHelp()
 {
 #if !defined(GUI) && !defined(SILENT)
   OutTitle();
   static MSGID Help[]={
 #ifdef SFX_MODULE
+    // Console SFX switches definition.
     MCHelpCmd,MSHelpCmdE,MSHelpCmdT,MSHelpCmdV
 #elif defined(UNRAR)
-    MUNRARTitle1,MRARTitle2,MCHelpCmd,MCHelpCmdE,MCHelpCmdL,MCHelpCmdP,
-    MCHelpCmdT,MCHelpCmdV,MCHelpCmdX,MCHelpSw,MCHelpSwm,MCHelpSwAC,MCHelpSwAD,
-    MCHelpSwAP,MCHelpSwAVm,MCHelpSwCm,MCHelpSwCFGm,MCHelpSwCL,MCHelpSwCU,
+    // UnRAR switches definition.
+    MUNRARTitle1,MRARTitle2,MCHelpCmd,MCHelpCmdE,MCHelpCmdL,
+    MCHelpCmdP,MCHelpCmdT,MCHelpCmdV,MCHelpCmdX,MCHelpSw,
+    MCHelpSwm,MCHelpSwAC,MCHelpSwAD,MCHelpSwAI,MCHelpSwAP,
+    MCHelpSwCm,MCHelpSwCFGm,MCHelpSwCL,MCHelpSwCU,
     MCHelpSwDH,MCHelpSwEP,MCHelpSwEP3,MCHelpSwF,MCHelpSwIDP,MCHelpSwIERR,
     MCHelpSwINUL,MCHelpSwIOFF,MCHelpSwKB,MCHelpSwN,MCHelpSwNa,MCHelpSwNal,
-    MCHelpSwOp,MCHelpSwOm,MCHelpSwOC,MCHelpSwOR,MCHelpSwOW,MCHelpSwP,
+    MCHelpSwO,MCHelpSwOC,MCHelpSwOR,MCHelpSwOW,MCHelpSwP,
     MCHelpSwPm,MCHelpSwR,MCHelpSwRI,MCHelpSwSL,MCHelpSwSM,MCHelpSwTA,
     MCHelpSwTB,MCHelpSwTN,MCHelpSwTO,MCHelpSwTS,MCHelpSwU,MCHelpSwVUnr,
     MCHelpSwVER,MCHelpSwVP,MCHelpSwX,MCHelpSwXa,MCHelpSwXal,MCHelpSwY
 #else
+    // RAR switches definition.
     MRARTitle1,MRARTitle2,MCHelpCmd,MCHelpCmdA,MCHelpCmdC,MCHelpCmdCF,
     MCHelpCmdCH,MCHelpCmdCW,MCHelpCmdD,MCHelpCmdE,MCHelpCmdF,MCHelpCmdI,
     MCHelpCmdK,MCHelpCmdL,MCHelpCmdM,MCHelpCmdP,MCHelpCmdR,MCHelpCmdRC,
     MCHelpCmdRN,MCHelpCmdRR,MCHelpCmdRV,MCHelpCmdS,MCHelpCmdT,MCHelpCmdU,
     MCHelpCmdV,MCHelpCmdX,MCHelpSw,MCHelpSwm,MCHelpSwAC,MCHelpSwAD,MCHelpSwAG,
-    MCHelpSwAO,MCHelpSwAP,MCHelpSwAS,MCHelpSwAV,MCHelpSwAVm,MCHelpSwCm,
-    MCHelpSwCFGm,MCHelpSwCL,MCHelpSwCU,MCHelpSwDF,MCHelpSwDH,MCHelpSwDS,
-    MCHelpSwEa,MCHelpSwED,MCHelpSwEE,MCHelpSwEN,MCHelpSwEP,MCHelpSwEP1,
-    MCHelpSwEP2,MCHelpSwEP3,MCHelpSwF,MCHelpSwHP,MCHelpSwIDP,
-    MCHelpSwIEML,MCHelpSwIERR,MCHelpSwILOG,MCHelpSwINUL,MCHelpSwIOFF,
-    MCHelpSwISND,MCHelpSwK,MCHelpSwKB,MCHelpSwMn,MCHelpSwMC,MCHelpSwMD,
-    MCHelpSwMS,MCHelpSwMT,MCHelpSwN,MCHelpSwNa,MCHelpSwNal,MCHelpSwOp,
-    MCHelpSwOm,MCHelpSwOC,MCHelpSwOL,MCHelpSwOR,MCHelpSwOS,MCHelpSwOW,
-    MCHelpSwP,MCHelpSwPm,MCHelpSwR,MCHelpSwR0,MCHelpSwRI,MCHelpSwRR,
-    MCHelpSwRV,MCHelpSwS,MCHelpSwSm,MCHelpSwSC,MCHelpSwSFX,MCHelpSwSI,
-    MCHelpSwSL,MCHelpSwSM,MCHelpSwT,MCHelpSwTA,MCHelpSwTB,MCHelpSwTK,
-    MCHelpSwTL,MCHelpSwTN,MCHelpSwTO,MCHelpSwTS,MCHelpSwU,MCHelpSwV,
-    MCHelpSwVn,MCHelpSwVD,MCHelpSwVER,MCHelpSwVN,MCHelpSwVP,MCHelpSwW,
-    MCHelpSwX,MCHelpSwXa,MCHelpSwXal,MCHelpSwY,MCHelpSwZ
+    MCHelpSwAI,MCHelpSwAO,MCHelpSwAP,MCHelpSwAS,MCHelpSwAV,MCHelpSwAVm,
+    MCHelpSwCm,MCHelpSwCFGm,MCHelpSwCL,MCHelpSwCU,MCHelpSwDF,MCHelpSwDH,
+    MCHelpSwDR,MCHelpSwDS,MCHelpSwDW,MCHelpSwEa,MCHelpSwED,MCHelpSwEE,
+    MCHelpSwEN,MCHelpSwEP,MCHelpSwEP1,MCHelpSwEP2,MCHelpSwEP3,MCHelpSwF,
+    MCHelpSwHP,MCHelpSwIDP,MCHelpSwIEML,MCHelpSwIERR,MCHelpSwILOG,MCHelpSwINUL,
+    MCHelpSwIOFF,MCHelpSwISND,MCHelpSwK,MCHelpSwKB,MCHelpSwMn,MCHelpSwMC,
+    MCHelpSwMD,MCHelpSwMS,MCHelpSwMT,MCHelpSwN,MCHelpSwNa,MCHelpSwNal,
+    MCHelpSwO,MCHelpSwOC,MCHelpSwOL,MCHelpSwOR,MCHelpSwOS,MCHelpSwOW,
+    MCHelpSwP,MCHelpSwPm,MCHelpSwR,MCHelpSwRm,MCHelpSwR0,MCHelpSwRI,
+    MCHelpSwRR,MCHelpSwRV,MCHelpSwS,MCHelpSwSm,MCHelpSwSC,MCHelpSwSFX,
+    MCHelpSwSI,MCHelpSwSL,MCHelpSwSM,MCHelpSwT,MCHelpSwTA,MCHelpSwTB,
+    MCHelpSwTK,MCHelpSwTL,MCHelpSwTN,MCHelpSwTO,MCHelpSwTS,MCHelpSwU,
+    MCHelpSwV,MCHelpSwVn,MCHelpSwVD,MCHelpSwVER,MCHelpSwVN,MCHelpSwVP,
+    MCHelpSwW,MCHelpSwX,MCHelpSwXa,MCHelpSwXal,MCHelpSwY,MCHelpSwZ
 #endif
   };
 
@@ -958,11 +989,11 @@ void CommandData::OutHelp()
 #ifndef _WIN_32
     static MSGID Win32Only[]={
       MCHelpSwIEML,MCHelpSwVD,MCHelpSwAO,MCHelpSwOS,MCHelpSwIOFF,
-      MCHelpSwEP2,MCHelpSwOC
+      MCHelpSwEP2,MCHelpSwOC,MCHelpSwDR,MCHelpSwRI
     };
     bool Found=false;
     for (int J=0;J<sizeof(Win32Only)/sizeof(Win32Only[0]);J++)
-      if (Help[I]==Win32Only[J])
+      if (CmpMSGID(Help[I],Win32Only[J]))
       {
         Found=true;
         break;
@@ -971,27 +1002,23 @@ void CommandData::OutHelp()
       continue;
 #endif
 #if !defined(_UNIX) && !defined(_WIN_32)
-    if (Help[I]==MCHelpSwOW)
+    if (CmpMSGID(Help[I],MCHelpSwOW))
       continue;
 #endif
 #if !defined(_WIN_32) && !defined(_EMX)
-    if (Help[I]==MCHelpSwAC)
+    if (CmpMSGID(Help[I],MCHelpSwAC))
       continue;
 #endif
 #ifndef SAVE_LINKS
-    if (Help[I]==MCHelpSwOL)
-      continue;
-#endif
-#ifndef _WIN_32
-    if (Help[I]==MCHelpSwRI)
+    if (CmpMSGID(Help[I],MCHelpSwOL))
       continue;
 #endif
 #ifndef PACK_SMP
-    if (Help[I]==MCHelpSwMT)
+    if (CmpMSGID(Help[I],MCHelpSwMT))
       continue;
 #endif
 #ifndef _BEOS
-    if (Help[I]==MCHelpSwEE)
+    if (CmpMSGID(Help[I],MCHelpSwEE))
     {
 #if defined(_EMX) && !defined(_DJGPP)
       if (_osmode != OS2_MODE)
@@ -1054,41 +1081,25 @@ bool CommandData::TimeCheck(RarTime &ft)
     return(true);
   if (FileTimeAfter.IsSet() && ft<=FileTimeAfter)
     return(true);
-/*
-  if (FileTimeOlder!=0 || FileTimeNewer!=0)
-  {
-    if (!TimeConverted)
-    {
-      if (FileTimeOlder!=0)
-        FileTimeOlder=SecondsToDosTime(FileTimeOlder);
-      if (FileTimeNewer!=0)
-        FileTimeNewer=SecondsToDosTime(FileTimeNewer);
-      TimeConverted=true;
-    }
-    if (FileTimeOlder!=0 && ft>=FileTimeOlder)
-      return(true);
-    if (FileTimeNewer!=0 && ft<=FileTimeNewer)
-      return(true);
-
-  }
-*/
   return(false);
 }
 #endif
 
 
 #ifndef SFX_MODULE
-bool CommandData::SizeCheck(Int64 Size)
+bool CommandData::SizeCheck(int64 Size)
 {
-  if (FileSizeLess!=INT64ERR && Size>=FileSizeLess)
+  if (FileSizeLess!=INT64NDF && Size>=FileSizeLess)
     return(true);
-  if (FileSizeMore!=INT64ERR && Size<=FileSizeMore)
+  if (FileSizeMore!=INT64NDF && Size<=FileSizeMore)
     return(true);
   return(false);
 }
 #endif
 
 
+
+
 int CommandData::IsProcessFile(FileHeader &NewLhd,bool *ExactMatch,int MatchType)
 {
   if (strlen(NewLhd.FileName)>=NM || strlenw(NewLhd.FileNameW)>=NM)
