@@ -1,20 +1,19 @@
 #ifndef _RAR_CMDDATA_
 #define _RAR_CMDDATA_
 
-#define DefaultStoreList "ace;arj;bz2;cab;gz;jpeg;jpg;lha;lzh;mp3;rar;zip;taz;tgz;z"
+#define DefaultStoreList "7z;ace;arj;bz2;cab;gz;jpeg;jpg;lha;lzh;mp3;rar;taz;tgz;z;zip"
 
 class CommandData:public RAROptions
 {
   private:
     void ProcessSwitchesString(char *Str);
-    void ProcessSwitch(char *Switch);
+    void ProcessSwitch(char *Switch,wchar *SwitchW=NULL);
     void BadSwitch(char *Switch);
-    bool ExclCheckArgs(StringList *Args,char *CheckName,bool CheckFullPath);
+    bool ExclCheckArgs(StringList *Args,char *CheckName,bool CheckFullPath,int MatchMode);
     uint GetExclAttr(char *Str);
 
     bool FileLists;
     bool NoMoreSwitches;
-    bool TimeConverted;
     bool BareOutput;
   public:
     CommandData();
@@ -32,6 +31,8 @@ class CommandData:public RAROptions
     bool ExclCheck(char *CheckName,bool CheckFullPath);
     bool StoreCheck(char *CheckName);
     bool TimeCheck(RarTime &ft);
+    bool SizeCheck(int64 Size);
+    bool AnyFiltersActive();
     int IsProcessFile(FileHeader &NewLhd,bool *ExactMatch=NULL,int MatchType=MATCH_WILDSUBPATH);
     void ProcessCommand();
     void AddArcName(char *Name,wchar *NameW);
