@@ -1,6 +1,6 @@
 #include "rar.hpp"
 
-MKDIR_CODE MakeDir(const char *Name,const wchar *NameW,uint Attr)
+MKDIR_CODE MakeDir(const char *Name,const wchar *NameW,bool SetAttr,uint Attr)
 {
 #ifdef _WIN_32
   int Success;
@@ -10,7 +10,8 @@ MKDIR_CODE MakeDir(const char *Name,const wchar *NameW,uint Attr)
     Success=CreateDirectory(Name,NULL);
   if (Success)
   {
-    SetFileAttr(Name,NameW,Attr);
+    if (SetAttr)
+      SetFileAttr(Name,NameW,Attr);
     return(MKDIR_SUCCESS);
   }
   int ErrCode=GetLastError();
@@ -25,15 +26,15 @@ MKDIR_CODE MakeDir(const char *Name,const wchar *NameW,uint Attr)
   if (__mkdir(Name)==0)
 #endif
   {
-    SetFileAttr(Name,NameW,Attr);
+    if (SetAttr)
+      SetFileAttr(Name,NameW,Attr);
     return(MKDIR_SUCCESS);
   }
   return(errno==ENOENT ? MKDIR_BADPATH:MKDIR_ERROR);
 #endif
 #ifdef _UNIX
-  int prevmask=umask(0);
-  int ErrCode=Name==NULL ? -1:mkdir(Name,(mode_t)Attr);
-  umask(prevmask);
+  mode_t uattr=SetAttr ? (mode_t)Attr:0777;
+  int ErrCode=Name==NULL ? -1:mkdir(Name,uattr);
   if (ErrCode==-1)
     return(errno==ENOENT ? MKDIR_BADPATH:MKDIR_ERROR);
   return(MKDIR_SUCCESS);
@@ -89,7 +90,7 @@ bool CreatePath(const char *Path,const wchar *PathW,bool SkipLastName)
         strncpy(DirName,Path,s-Path);
         DirName[s-Path]=0;
       }
-      if (MakeDir(DirName,DirPtrW,DirAttr)==MKDIR_SUCCESS)
+      if (MakeDir(DirName,DirPtrW,true,DirAttr)==MKDIR_SUCCESS)
       {
 #ifndef GUI
         mprintf(St(MCreatDir),DirName);
@@ -103,13 +104,13 @@ bool CreatePath(const char *Path,const wchar *PathW,bool SkipLastName)
       s=charnext(s);
   }
   if (!SkipLastName && !IsPathDiv(*PointToLastChar(Path)))
-    if (MakeDir(Path,PathW,DirAttr)!=MKDIR_SUCCESS)
+    if (MakeDir(Path,PathW,true,DirAttr)!=MKDIR_SUCCESS)
       Success=false;
   return(Success);
 }
 
 
-void SetDirTime(const char *Name,RarTime *ftm,RarTime *ftc,RarTime *fta)
+void SetDirTime(const char *Name,const wchar *NameW,RarTime *ftm,RarTime *ftc,RarTime *fta)
 {
 #ifdef _WIN_32
   if (!WinNT())
@@ -119,11 +120,14 @@ void SetDirTime(const char *Name,RarTime *ftm,RarTime *ftc,RarTime *fta)
   bool sc=ftc!=NULL && ftc->IsSet();
   bool sa=fta!=NULL && fta->IsSet();
 
-  unsigned int DirAttr=GetFileAttr(Name);
+  unsigned int DirAttr=GetFileAttr(Name,NameW);
   bool ResetAttr=(DirAttr!=0xffffffff && (DirAttr & FA_RDONLY)!=0);
   if (ResetAttr)
-    SetFileAttr(Name,NULL,0);
-  HANDLE hFile=CreateFile(Name,GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,
+    SetFileAttr(Name,NameW,0);
+
+  wchar DirNameW[NM];
+  GetWideName(Name,NameW,DirNameW);
+  HANDLE hFile=CreateFileW(DirNameW,GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,
                           NULL,OPEN_EXISTING,FILE_FLAG_BACKUP_SEMANTICS,NULL);
   if (hFile==INVALID_HANDLE_VALUE)
     return;
@@ -137,7 +141,7 @@ void SetDirTime(const char *Name,RarTime *ftm,RarTime *ftc,RarTime *fta)
   SetFileTime(hFile,sc ? &fc:NULL,sa ? &fa:NULL,sm ? &fm:NULL);
   CloseHandle(hFile);
   if (ResetAttr)
-    SetFileAttr(Name,NULL,DirAttr);
+    SetFileAttr(Name,NameW,DirAttr);
 #endif
 #if defined(_UNIX) || defined(_EMX)
   File::SetCloseFileTimeByName(Name,ftm,fta);
@@ -162,7 +166,7 @@ bool IsRemovable(const char *Name)
 
 
 #ifndef SFX_MODULE
-Int64 GetFreeDisk(const char *Name)
+int64 GetFreeDisk(const char *Name)
 {
 #ifdef _WIN_32
   char Root[NM];
@@ -186,13 +190,14 @@ Int64 GetFreeDisk(const char *Name)
     uiUserFree.u.LowPart=uiUserFree.u.HighPart=0;
     if (pGetDiskFreeSpaceEx(*Root ? Root:NULL,&uiUserFree,&uiTotalSize,&uiTotalFree) &&
         uiUserFree.u.HighPart<=uiTotalFree.u.HighPart)
-      return(int32to64(uiUserFree.u.HighPart,uiUserFree.u.LowPart));
+      return(INT32TO64(uiUserFree.u.HighPart,uiUserFree.u.LowPart));
   }
 
+  // We are here if we failed to load GetDiskFreeSpaceExA.
   DWORD SectorsPerCluster,BytesPerSector,FreeClusters,TotalClusters;
   if (!GetDiskFreeSpace(*Root ? Root:NULL,&SectorsPerCluster,&BytesPerSector,&FreeClusters,&TotalClusters))
     return(1457664);
-  Int64 FreeSize=SectorsPerCluster*BytesPerSector;
+  int64 FreeSize=SectorsPerCluster*BytesPerSector;
   FreeSize=FreeSize*FreeClusters;
   return(FreeSize);
 #elif defined(_BEOS)
@@ -204,7 +209,7 @@ Int64 GetFreeDisk(const char *Name)
   fs_info Info;
   if (fs_stat_dev(Dev,&Info)!=0)
     return(1457664);
-  Int64 FreeSize=Info.block_size;
+  int64 FreeSize=Info.block_size;
   FreeSize=FreeSize*Info.free_blocks;
   return(FreeSize);
 #elif defined(_UNIX)
@@ -217,7 +222,7 @@ Int64 GetFreeDisk(const char *Name)
     FSALLOCATE fsa;
     if (DosQueryFSInfo(Drive,1,&fsa,sizeof(fsa))!=0)
       return(1457664);
-    Int64 FreeSize=fsa.cSectorUnit*fsa.cbSector;
+    int64 FreeSize=fsa.cSectorUnit*fsa.cbSector;
     FreeSize=FreeSize*fsa.cUnitAvail;
     return(FreeSize);
   }
@@ -235,7 +240,7 @@ Int64 GetFreeDisk(const char *Name)
 #endif
     if (outregs.x.ax==0xffff)
       return(1457664);
-    Int64 FreeSize=outregs.x.ax*outregs.x.cx;
+    int64 FreeSize=outregs.x.ax*outregs.x.cx;
     FreeSize=FreeSize*outregs.x.bx;
     return(FreeSize);
   }
@@ -402,8 +407,10 @@ void ConvertNameToFull(const char *Src,char *Dest)
     strcpy(FullName,Src);
   else
   {
-    getcwd(FullName,sizeof(FullName));
-    AddEndSlash(FullName);
+    if (getcwd(FullName,sizeof(FullName))==NULL)
+      *FullName=0;
+    else
+      AddEndSlash(FullName);
     strcat(FullName,Src);
   }
   strcpy(Dest,FullName);
@@ -455,7 +462,7 @@ void ConvertNameToFull(const wchar *Src,wchar *Dest)
 #ifndef SFX_MODULE
 char *MkTemp(char *Name)
 {
-  int Length=strlen(Name);
+  size_t Length=strlen(Name);
   if (Length<=6)
     return(NULL);
   int Random=clock();
@@ -476,17 +483,16 @@ char *MkTemp(char *Name)
 
 
 #ifndef SFX_MODULE
-uint CalcFileCRC(File *SrcFile,Int64 Size,CALCCRC_SHOWMODE ShowMode)
+uint CalcFileCRC(File *SrcFile,int64 Size,CALCCRC_SHOWMODE ShowMode)
 {
   SaveFilePos SavePos(*SrcFile);
-  const int BufSize=0x10000;
+  const size_t BufSize=0x10000;
   Array<byte> Data(BufSize);
-  Int64 BlockCount=0;
+  int64 BlockCount=0;
   uint DataCRC=0xffffffff;
-  int ReadSize;
 
 #if !defined(SILENT) && !defined(_WIN_CE)
-  Int64 FileLength=SrcFile->FileLength();
+  int64 FileLength=SrcFile->FileLength();
   if (ShowMode!=CALCCRC_SHOWNONE)
   {
     mprintf(St(MCalcCRC));
@@ -496,19 +502,28 @@ uint CalcFileCRC(File *SrcFile,Int64 Size,CALCCRC_SHOWMODE ShowMode)
 #endif
 
   SrcFile->Seek(0,SEEK_SET);
-  while ((ReadSize=SrcFile->Read(&Data[0],int64to32(Size==INT64ERR ? Int64(BufSize):Min(Int64(BufSize),Size))))!=0)
+  while (true)
   {
+    size_t SizeToRead;
+    if (Size==INT64NDF)   // If we process the entire file.
+      SizeToRead=BufSize; // Then always attempt to read the entire buffer.
+    else
+      SizeToRead=(size_t)Min((int64)BufSize,Size);
+    int ReadSize=SrcFile->Read(&Data[0],SizeToRead);
+    if (ReadSize==0)
+      break;
+
     ++BlockCount;
     if ((BlockCount & 15)==0)
     {
 #if !defined(SILENT) && !defined(_WIN_CE)
       if (ShowMode==CALCCRC_SHOWALL)
-        mprintf("\b\b\b\b%3d%%",ToPercent(BlockCount*Int64(BufSize),FileLength));
+        mprintf("\b\b\b\b%3d%%",ToPercent(BlockCount*int64(BufSize),FileLength));
 #endif
       Wait();
     }
     DataCRC=CRC(DataCRC,&Data[0],ReadSize);
-    if (Size!=INT64ERR)
+    if (Size!=INT64NDF)
       Size-=ReadSize;
   }
 #if !defined(SILENT) && !defined(_WIN_CE)
@@ -538,19 +553,11 @@ bool DelFile(const char *Name,const wchar *NameW)
 }
 
 
-bool DelDir(const char *Name)
-{
-  return(DelDir(Name,NULL));
-}
 
 
-bool DelDir(const char *Name,const wchar *NameW)
-{
-  return(rmdir(Name)==0);
-}
 
 
-#if defined(_WIN_32) && !defined(_WIN_CE)
+#if defined(_WIN_32) && !defined(_WIN_CE) && !defined(SFX_MODULE)
 bool SetFileCompression(char *Name,wchar *NameW,bool State)
 {
   wchar FileNameW[NM];
@@ -568,3 +575,11 @@ bool SetFileCompression(char *Name,wchar *NameW,bool State)
   return(RetCode!=0);
 }
 #endif
+
+
+
+
+
+
+
+
