@@ -2,12 +2,13 @@
 
 static bool IsUnicode(byte *Data,int Size);
 
-bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
-                  bool ConvertToAnsi,bool Unquote,bool SkipComments)
+bool ReadTextFile(const char *Name,StringList *List,bool Config,
+                  bool AbortOnError,RAR_CHARSET SrcCharset,bool Unquote,
+                  bool SkipComments,bool ExpandEnvStr)
 {
   char FileName[NM];
   if (Config)
-    GetConfigName(Name,FileName);
+    GetConfigName(Name,FileName,true);
   else
     strcpy(FileName,Name);
 
@@ -37,9 +38,19 @@ bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
 
   memset(&Data[DataSize],0,5);
 
-  if (IsUnicode((byte *)&Data[0],DataSize))
+  if (SrcCharset==RCH_UNICODE ||
+      SrcCharset==RCH_DEFAULT && IsUnicode((byte *)&Data[0],DataSize))
   {
-    wchar *CurStr=(wchar *)&Data[2];
+    // Unicode in native system format, can be more than 2 bytes per character.
+    Array<wchar> DataW(Data.Size()/2+1);
+    for (size_t I=2;I<Data.Size()-1;I+=2)
+    {
+      // Need to convert Data to (byte) first to prevent the sign extension
+      // to higher bytes.
+      DataW[(I-2)/2]=(wchar)((byte)Data[I])+(wchar)((byte)Data[I+1])*256;
+    }
+
+    wchar *CurStr=&DataW[0];
     Array<char> AnsiName;
 
     while (*CurStr!=0)
@@ -63,8 +74,11 @@ bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
       }
       if (*CurStr)
       {
-        int Length=strlenw(CurStr);
-        int AddSize=Length-AnsiName.Size()+1;
+        // Length and AddSize must be defined as signed, because AddSize
+        // can be negative.
+        int Length=(int)strlenw(CurStr);
+        int AddSize=4*(Length-(int)AnsiName.Size()+1);
+
         if (AddSize>0)
           AnsiName.Add(AddSize);
         if (Unquote && *CurStr=='\"' && CurStr[Length-1]=='\"')
@@ -72,8 +86,29 @@ bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
           CurStr[Length-1]=0;
           CurStr++;
         }
-        WideToChar(CurStr,&AnsiName[0]);
-        List->AddString(&AnsiName[0],CurStr);
+        WideToChar(CurStr,&AnsiName[0],AnsiName.Size());
+
+        bool Expanded=false;
+#if defined(_WIN_32) && !defined(_WIN_CE)
+        if (ExpandEnvStr && *CurStr=='%')
+        {
+          // expanding environment variables in Windows version
+
+          char ExpName[NM];
+          wchar ExpNameW[NM];
+          *ExpNameW=0;
+          int ret,retw=1;
+          ret=ExpandEnvironmentStrings(&AnsiName[0],ExpName,ASIZE(ExpName));
+          if (ret!=0 && WinNT())
+            retw=ExpandEnvironmentStringsW(CurStr,ExpNameW,ASIZE(ExpNameW));
+          Expanded=ret!=0 && ret<ASIZE(ExpName) &&
+                   retw!=0 && retw<ASIZE(ExpNameW);
+          if (Expanded)
+            List->AddString(ExpName,ExpNameW);
+        }
+#endif
+        if (!Expanded)
+          List->AddString(&AnsiName[0],CurStr);
       }
       CurStr=NextStr+1;
       while (*CurStr=='\r' || *CurStr=='\n')
@@ -106,7 +141,7 @@ bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
       {
         if (Unquote && *CurStr=='\"')
         {
-          int Length=strlen(CurStr);
+          size_t Length=strlen(CurStr);
           if (CurStr[Length-1]=='\"')
           {
             CurStr[Length-1]=0;
@@ -114,10 +149,25 @@ bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
           }
         }
 #if defined(_WIN_32)
-        if (ConvertToAnsi)
+        if (SrcCharset==RCH_OEM)
           OemToChar(CurStr,CurStr);
 #endif
-        List->AddString(CurStr);
+
+        bool Expanded=false;
+#if defined(_WIN_32) && !defined(_WIN_CE)
+        if (ExpandEnvStr && *CurStr=='%')
+        {
+          // expanding environment variables in Windows version
+
+          char ExpName[NM];
+          int ret=ExpandEnvironmentStrings(CurStr,ExpName,ASIZE(ExpName));
+          Expanded=ret!=0 && ret<ASIZE(ExpName);
+          if (Expanded)
+            List->AddString(ExpName);
+        }
+#endif
+        if (!Expanded)
+          List->AddString(CurStr);
       }
       CurStr=NextStr+1;
       while (*CurStr=='\r' || *CurStr=='\n')
@@ -133,7 +183,7 @@ bool IsUnicode(byte *Data,int Size)
   if (Size<4 || Data[0]!=0xff || Data[1]!=0xfe)
     return(false);
   for (int I=2;I<Size;I++)
-    if (Data[I]<32)
+    if (Data[I]<32 && Data[I]!='\r' && Data[I]!='\n')
       return(true);
   return(false);
 }
