@@ -1,13 +1,5 @@
 #include "rar.hpp"
 
-
-#include "smallfn.cpp"
-
-#ifdef _DJGPP
-extern "C" char **__crt0_glob_function (char *arg) { return 0; }
-extern "C" void   __crt0_load_environment_file (char *progname) { }
-#endif
-
 #if !defined(GUI) && !defined(RARDLL)
 int main(int argc, char *argv[])
 {
@@ -46,6 +38,9 @@ int main(int argc, char *argv[])
 
 #endif
 
+#if defined(_WIN_32) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
+  bool ShutdownOnClose;
+#endif
 
 #ifdef ALLOW_EXCEPTIONS
   try 
@@ -95,6 +90,9 @@ int main(int argc, char *argv[])
 #endif
     Cmd.ParseDone();
 
+#if defined(_WIN_32) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
+    ShutdownOnClose=Cmd.Shutdown;
+#endif
 
     InitConsoleOptions(Cmd.MsgStream,Cmd.Sound);
     InitLogOptions(Cmd.LogName);
@@ -126,6 +124,10 @@ int main(int argc, char *argv[])
 #endif
 #if defined(_EMX) && !defined(_DJGPP)
   uni_done();
+#endif
+#if defined(_WIN_32) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
+  if (ShutdownOnClose)
+    Shutdown();
 #endif
   return(ErrHandler.GetErrorCode());
 }
