@@ -4,11 +4,11 @@
 #include "unios2.cpp"
 #endif
 
-bool WideToChar(const wchar *Src,char *Dest,int DestSize)
+bool WideToChar(const wchar *Src,char *Dest,size_t DestSize)
 {
   bool RetCode=true;
 #ifdef _WIN_32
-  if (WideCharToMultiByte(CP_ACP,0,Src,-1,Dest,DestSize,NULL,NULL)==0)
+  if (WideCharToMultiByte(CP_ACP,0,Src,-1,Dest,(int)DestSize,NULL,NULL)==0)
     RetCode=false;
 #else
 #ifdef _APPLE
@@ -57,11 +57,11 @@ bool WideToChar(const wchar *Src,char *Dest,int DestSize)
 }
 
 
-bool CharToWide(const char *Src,wchar *Dest,int DestSize)
+bool CharToWide(const char *Src,wchar *Dest,size_t DestSize)
 {
   bool RetCode=true;
 #ifdef _WIN_32
-  if (MultiByteToWideChar(CP_ACP,0,Src,-1,Dest,DestSize)==0)
+  if (MultiByteToWideChar(CP_ACP,0,Src,-1,Dest,(int)DestSize)==0)
     RetCode=false;
 #else
 #ifdef _APPLE
@@ -109,9 +109,9 @@ bool CharToWide(const char *Src,wchar *Dest,int DestSize)
 }
 
 
-byte* WideToRaw(const wchar *Src,byte *Dest,int DestSize)
+byte* WideToRaw(const wchar *Src,byte *Dest,size_t DestSize)
 {
-  for (int I=0;I<DestSize;I++,Src++)
+  for (size_t I=0;I<DestSize;I++,Src++)
   {
     Dest[I*2]=(byte)*Src;
     Dest[I*2+1]=(byte)(*Src>>8);
@@ -122,38 +122,39 @@ byte* WideToRaw(const wchar *Src,byte *Dest,int DestSize)
 }
 
 
-wchar* RawToWide(const byte *Src,wchar *Dest,int DestSize)
+wchar* RawToWide(const byte *Src,wchar *Dest,size_t DestSize)
 {
-  for (int I=0;I<DestSize;I++)
+  for (size_t I=0;I<DestSize;I++)
     if ((Dest[I]=Src[I*2]+(Src[I*2+1]<<8))==0)
       break;
   return(Dest);
 }
 
 
-void WideToUtf(const wchar *Src,char *Dest,int DestSize)
+void WideToUtf(const wchar *Src,char *Dest,size_t DestSize)
 {
-  DestSize--;
-  while (*Src!=0 && --DestSize>=0)
+  long dsize=(long)DestSize;
+  dsize--;
+  while (*Src!=0 && --dsize>=0)
   {
     uint c=*(Src++);
     if (c<0x80)
       *(Dest++)=c;
     else
-      if (c<0x800 && --DestSize>=0)
+      if (c<0x800 && --dsize>=0)
       {
         *(Dest++)=(0xc0|(c>>6));
         *(Dest++)=(0x80|(c&0x3f));
       }
       else
-        if (c<0x10000 && (DestSize-=2)>=0)
+        if (c<0x10000 && (dsize-=2)>=0)
         {
           *(Dest++)=(0xe0|(c>>12));
           *(Dest++)=(0x80|((c>>6)&0x3f));
           *(Dest++)=(0x80|(c&0x3f));
         }
         else
-          if (c < 0x200000 && (DestSize-=3)>=0)
+          if (c < 0x200000 && (dsize-=3)>=0)
           {
             *(Dest++)=(0xf0|(c>>18));
             *(Dest++)=(0x80|((c>>12)&0x3f));
@@ -165,9 +166,10 @@ void WideToUtf(const wchar *Src,char *Dest,int DestSize)
 }
 
 
-void UtfToWide(const char *Src,wchar *Dest,int DestSize)
+void UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
 {
-  DestSize--;
+  long dsize=(long)DestSize;
+  dsize--;
   while (*Src!=0)
   {
     uint c=(byte)*(Src++),d;
@@ -199,11 +201,11 @@ void UtfToWide(const char *Src,wchar *Dest,int DestSize)
           }
           else
             break;
-    if (--DestSize<0)
+    if (--dsize<0)
       break;
     if (d>0xffff)
     {
-      if (--DestSize<0 || d>0x10ffff)
+      if (--dsize<0 || d>0x10ffff)
         break;
       *(Dest++)=((d-0x10000)>>10)+0xd800;
       *(Dest++)=(d&0x3ff)+0xdc00;
@@ -229,9 +231,9 @@ bool UnicodeEnabled()
 }
 
 
-int strlenw(const wchar *str)
+size_t strlenw(const wchar *str)
 {
-  int length=0;
+  size_t length=0;
   while (*(str++)!=0)
     length++;
   return(length);
@@ -359,7 +361,7 @@ wchar* strchrw(const wchar *s,int c)
 
 wchar* strrchrw(const wchar *s,int c)
 {
-  for (int I=strlenw(s)-1;I>=0;I--)
+  for (int I=(int)(strlenw(s)-1);I>=0;I--)
     if (s[I]==c)
       return((wchar *)(s+I));
   return(NULL);
@@ -383,7 +385,7 @@ wchar* strlowerw(wchar *Str)
 {
   for (wchar *ChPtr=Str;*ChPtr;ChPtr++)
     if (*ChPtr<128)
-      *ChPtr=loctolower(*ChPtr);
+      *ChPtr=loctolower((byte)*ChPtr);
   return(Str);
 }
 #endif
@@ -394,7 +396,7 @@ wchar* strupperw(wchar *Str)
 {
   for (wchar *ChPtr=Str;*ChPtr;ChPtr++)
     if (*ChPtr<128)
-      *ChPtr=loctoupper(*ChPtr);
+      *ChPtr=loctoupper((byte)*ChPtr);
   return(Str);
 }
 #endif
@@ -448,8 +450,8 @@ void SupportDBCS::Init()
   CPINFO CPInfo;
   GetCPInfo(CP_ACP,&CPInfo);
   DBCSMode=CPInfo.MaxCharSize > 1;
-  for (int I=0;I<sizeof(IsLeadByte)/sizeof(IsLeadByte[0]);I++)
-    IsLeadByte[I]=IsDBCSLeadByte(I);
+  for (uint I=0;I<ASIZE(IsLeadByte);I++)
+    IsLeadByte[I]=IsDBCSLeadByte(I)!=0;
 }
 
 
@@ -459,9 +461,9 @@ char* SupportDBCS::charnext(const char *s)
 }
 
 
-uint SupportDBCS::strlend(const char *s)
+size_t SupportDBCS::strlend(const char *s)
 {
-  uint Length=0;
+  size_t Length=0;
   while (*s!=0)
   {
     if (IsLeadByte[*s])
